@@ -38,7 +38,11 @@ struct _GlJournalEntry
   gchar *transport;
   gchar *catalog;
   guint priority;
-  gint uid;
+  gchar *uid;
+  gchar *pid;
+  gchar *gid;
+  gchar *systemd_unit;
+  gchar *executable_path;
 };
 
 G_DEFINE_TYPE (GlJournalEntry, gl_journal_entry, G_TYPE_OBJECT);
@@ -363,7 +367,6 @@ _gl_journal_query_entry (GlJournal *self)
     sd_journal *journal;
     GError *error = NULL;
     gchar *priority;
-    gchar *uid;
 
     priv = gl_journal_get_instance_private (self);
     journal = priv->journal;
@@ -479,7 +482,7 @@ _gl_journal_query_entry (GlJournal *self)
         g_clear_error (&error);
     }
 
-    uid = gl_journal_get_data (self, "_UID", &error);
+    entry->uid = gl_journal_get_data (self, "_UID", &error);
 
     if (error != NULL)
     {
@@ -487,9 +490,37 @@ _gl_journal_query_entry (GlJournal *self)
         g_clear_error (&error);
     }
 
-    /* We store an invalid or non-existent UID as -1 */
-    entry->uid = uid ? atoi (uid) : -1;
-    g_free (uid);
+    entry->pid = gl_journal_get_data (self, "_PID", &error);
+
+    if (error != NULL)
+    {
+        g_debug ("Error while getting pid from journal: %s", error->message);
+        g_clear_error (&error);
+    }
+
+    entry->gid = gl_journal_get_data (self, "_GID", &error);
+
+    if (error != NULL)
+    {
+        g_debug ("Error while getting gid from journal: %s", error->message);
+        g_clear_error (&error);
+    }
+
+    entry->systemd_unit = gl_journal_get_data (self, "_SYSTEMD_UNIT", &error);
+
+    if (error != NULL)
+    {
+        g_debug ("Error while getting systemd unit from journal: %s", error->message);
+        g_clear_error (&error);
+    }
+
+    entry->executable_path = gl_journal_get_data (self, "_EXE", &error);
+
+    if (error != NULL)
+    {
+        g_debug ("Error while getting executable path from journal: %s", error->message);
+        g_clear_error (&error);
+    }
 
     return entry;
 
@@ -565,8 +596,8 @@ create_boot_id_match_string (void)
  * beginning.
  */
 void
-gl_journal_set_matches (GlJournal           *journal,
-                        const gchar * const *matches)
+gl_journal_set_matches (GlJournal *journal,
+                        GPtrArray *matches)
 {
     GlJournalPrivate *priv = gl_journal_get_instance_private (journal);
     GPtrArray *mandatory_fields;
@@ -582,25 +613,29 @@ gl_journal_set_matches (GlJournal           *journal,
     sd_journal_flush_matches (priv->journal);
 
     mandatory_fields = g_ptr_array_new ();
-    for (i = 0; matches[i]; i++)
+    for (i = 0; i < matches->len; i++)
     {
         /* matches without a value should only check for existence.
          * systemd doesn't support that, so let's remember them to
          * filter out later.
          */
-        if (strchr (matches[i], '=') == NULL)
+        const gchar *match;
+
+        match = g_ptr_array_index (matches, i);
+
+        if (strchr (match, '=') == NULL)
         {
-            g_ptr_array_add (mandatory_fields, g_strdup (matches[i]));
+            g_ptr_array_add (mandatory_fields, g_strdup (match));
             continue;
         }
 
-        if (g_str_has_prefix (matches[i], "_BOOT_ID="))
+        if (g_str_has_prefix (match, "_BOOT_ID="))
           has_boot_id = TRUE;
 
-        r = sd_journal_add_match (priv->journal, matches[i], 0);
+        r = sd_journal_add_match (priv->journal, match, 0);
         if (r < 0)
         {
-            g_critical ("Failed to add match '%s': %s", matches[i], g_strerror (-r));
+            g_critical ("Failed to add match '%s': %s", match, g_strerror (-r));
             break;
         }
     }
@@ -611,16 +646,16 @@ gl_journal_set_matches (GlJournal           *journal,
     /* take events from this boot only, unless _BOOT_ID was in @matches */
     if (!has_boot_id)
     {
-        gchar *match;
+        gchar *boot_match;
 
-        match = create_boot_id_match_string ();
-        if (match)
+        boot_match = create_boot_id_match_string ();
+        if (boot_match)
         {
-            r = sd_journal_add_match (priv->journal, match, 0);
+            r = sd_journal_add_match (priv->journal, boot_match, 0);
             if (r < 0)
-                g_warning ("Failed to add match '%s': %s", matches[i], g_strerror (-r));
+                g_warning ("Failed to add match '%s': %s", boot_match, g_strerror (-r));
 
-            g_free (match);
+            g_free (boot_match);
         }
     }
 
@@ -677,6 +712,11 @@ gl_journal_entry_finalize (GObject *object)
   g_free (entry->kernel_device);
   g_free (entry->audit_session);
   g_free (entry->transport);
+  g_free (entry->uid);
+  g_free (entry->pid);
+  g_free (entry->gid);
+  g_free (entry->systemd_unit);
+  g_free (entry->executable_path);
 
   G_OBJECT_CLASS (gl_journal_entry_parent_class)->finalize (object);
 }
@@ -753,10 +793,42 @@ gl_journal_entry_get_priority (GlJournalEntry *entry)
   return entry->priority;
 }
 
-gint
+const gchar *
 gl_journal_entry_get_uid (GlJournalEntry *entry)
 {
-  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), -1);
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
 
   return entry->uid;
 }
+
+const gchar *
+gl_journal_entry_get_pid (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
+
+  return entry->pid;
+}
+
+const gchar *
+gl_journal_entry_get_gid (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
+
+  return entry->gid;
+}
+
+const gchar *
+gl_journal_entry_get_systemd_unit (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
+
+  return entry->systemd_unit;
+}
+
+const gchar *
+gl_journal_entry_get_executable_path (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
+
+  return entry->executable_path;
+}
